@@ -1,10 +1,16 @@
 import collections
 import copy
+import logging
 import new
+import traceback
+import warnings
 
 from operator import itemgetter
 
 
+logger = logging.getLogger(__name__)
+
+
 class EnumConstructionException(Exception):
     """
     Raised whenever there's an error in an Enum's declaration.
@@ -19,6 +25,13 @@ class EnumLookupError(Exception):
     pass
 
 
+class EnumComparisonWarning(Warning):
+    """
+    Raised when comparing an enum to a value of another type.
+    """
+    pass
+
+
 def enum(**enums):
     """
     A basic enum implementation.
@@ -66,18 +79,31 @@ class RichEnumValue(object):
         return hash(self.canonical_name)
 
     def __cmp__(self, other):
+        if other is None:
+            return -1
         if not isinstance(other, type(self)):
+            self._warn_about_compare(other)
             return -1
         return cmp(self.canonical_name, other.canonical_name)
 
     def __eq__(self, other):
+        if other is None:
+            return False
         if not isinstance(other, type(self)):
+            self._warn_about_compare(other)
             return False
         return self.canonical_name == other.canonical_name
 
     def __ne__(self, other):
         return not (self.__eq__(other))
 
+    def _warn_about_compare(self, other):
+        warnings.warn(
+            'Comparing a %s to a %s!' % (type(other), type(self)),
+            EnumComparisonWarning,
+            stacklevel=3)  # Complain about _warn_about_compare's caller's caller
+        logger.debug(''.join(traceback.format_stack()))
+
     def choicify(self, value_field="canonical_name", display_field="display_name"):
         """
         DEPRECATED
@@ -106,18 +132,21 @@ class OrderedRichEnumValue(RichEnumValue):
                           self.display_name))
 
     def __cmp__(self, other):
+        if other is None:
+            return -1
         if not isinstance(other, type(self)):
+            self._warn_about_compare(other)
             return -1
         return cmp(self.index, other.index)
 
     def __eq__(self, other):
+        if other is None:
+            return False
         if not isinstance(other, type(self)):
+            self._warn_about_compare(other)
             return False
         return self.index == other.index
 
-    def __ne__(self, other):
-        return not (self.__eq__(other))
-
 
 def _setup_members(cls_attrs, cls_parents, member_cls):
         members = []
@@ -125,6 +154,7 @@ def _setup_members(cls_attrs, cls_parents, member_cls):
         # Find qualified "EnumValue" attributes.
         # Field names must be UPPERCASE, not prefixed with _ ("internal"),
         # and values must be EnumValue-derived.
+        last_type = None
         for attr_key, attr_value in cls_attrs.items():
             # Skip "internal" attributes
             if attr_key.startswith("_"):
@@ -138,6 +168,13 @@ def _setup_members(cls_attrs, cls_parents, member_cls):
             else:
                 raise EnumConstructionException("Invalid attribute: %s" % attr_key)
 
+            attr_type = type(attr_value)
+            if last_type and attr_type != last_type:
+                raise EnumConstructionException("Differing member types: have seen %s,"
+                                                " encountered %s" % (last_type, attr_type))
+            else:
+                last_type = attr_type
+
         if cls_parents not in [(object, ), (_EnumMethods, )] and not members:
             raise EnumConstructionException(
                 "Must specify at least one attribute when using RichEnum")
@@ -153,6 +190,15 @@ class _BaseRichEnumMetaclass(type):
     def __len__(cls):
         return len(cls._MEMBERS)
 
+    def __contains__(cls, item):
+        # Check membership without comparing enum values to other types.
+        members = cls.members()
+        if not members:
+            return False
+        if not type(members[0]) == type(item):
+            return False
+        return (item in members)
+
 
 class _RichEnumMetaclass(_BaseRichEnumMetaclass):
     def __new__(cls, cls_name, cls_parents, cls_attrs):
@@ -196,8 +242,18 @@ class _EnumMethods(object):
     def lookup(cls, field, value):
         for member in cls:
             member_value = getattr(member, field)
-            if member_value == value:
-                return member
+
+            # We filter warnings below; in the case where the field is a list
+            # of RichEnumValues and the value we're looking for is a RichEnum,
+            # this generates a warning due to mismatched type comparison even
+            # though the calling pattern is acceptable.
+            # n.b. There's likely a cleaner way to fix this but I was unsure if
+            # doing so would break the semantics of this method.
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore')
+                if member_value == value:
+                    return member
+
             if (
                 not isinstance(member_value, str) and
                 not isinstance(member_value, unicode) and
